/********************************************************************************
*										*
*	Program  ::mpi_example16.c						*
*	Description :: Creating Groups and Communicator				*
*********************************************************************************/ #include <stdio.h>
#include <stdlib.h>
#include <mpi.h>
#include <math.h>


int main(argc,argv)
int argc;
char *argv[];
{
	int rank, numprocs,i,group1_rank,group2_rank,group3_rank,group4_rank;
	int group1_size,group2_size,group3_size,group4_size,flag;
	
	MPI_Comm comm1,comm2,comm3,comm4;
	MPI_Group group1,group2,group3,group4,main_group;

	MPI_Init(&argc,&argv);
	MPI_Comm_size(MPI_COMM_WORLD,&numprocs);
	MPI_Comm_rank(MPI_COMM_WORLD,&rank);
    	
	MPI_Comm_group(MPI_COMM_WORLD,&main_group);
	
	int ranks[numprocs];

	for (i=0;i<numprocs;i++) {
        	ranks[i] = i;     //For forming groups, later
    	}
	
		//creating group2 with 3 ranks from comm_world's group
		MPI_Group_incl(main_group,3,ranks+5,&group2);
		MPI_Comm_create(MPI_COMM_WORLD, group2, &comm2);
		MPI_Group_size(group2,&group2_size);
		MPI_Group_rank(group2, &group2_rank);
	

		//creating group1 with 3 ranks from  comm_world's group
		MPI_Group_incl(main_group,3,ranks,&group1);
		MPI_Comm_create(MPI_COMM_WORLD, group1, &comm1);
		MPI_Group_size(group1,&group1_size);
		MPI_Group_rank(group1, &group1_rank);
	
		//creating group3 with 2 ranks from  comm_world's group
		MPI_Group_incl(main_group,2,ranks+3,&group3);
		MPI_Comm_create(MPI_COMM_WORLD, group3, &comm3);
		MPI_Group_size(group3,&group3_size);
		MPI_Group_rank(group3, &group3_rank);

		//printf("1rank = %d\n",rank);
		//creating inter communicator comm4 for group1 and group3
		if(comm1 != MPI_COMM_NULL)
		{
			//printf("2rank = %d\n",rank);
			 MPI_Intercomm_create(comm1, 0, MPI_COMM_WORLD,3,20, &comm4); 
			
		}
		if(comm3 != MPI_COMM_NULL)
		{
			//printf("3rank = %d\n",rank);
		 	MPI_Intercomm_create(comm3, 0, MPI_COMM_WORLD, 0, 20, &comm4); 
			
		}

		//printf("4rank = %d\n",rank);
		if(comm4 != MPI_COMM_NULL && comm3 != MPI_COMM_NULL)
		{
			printf("rank = %d\n",rank);
			MPI_Comm_test_inter(comm4,&flag); //testing comm4 is inter communicator or intra communicator if flag =1 then inter communicator
			printf("flag = %d\n",flag);

			MPI_Comm_free(&comm4);
		}
	
	

	if(rank==1)
	{
		printf("******* size of comm_world=%d, group1_size=%d, group2_size=%d, group3_size=%d ******* \n",numprocs, group1_size,group2_size,group3_size);
		fflush(0);
		printf("\n");
	}
	
	MPI_Barrier(MPI_COMM_WORLD);


	printf("rank=%d, group1_rank=%d, group2_rank=%d, group3_rank=%d\n", rank,group1_rank, group2_rank, group3_rank);
	fflush(0);
	printf("\n");

	
	if(comm1 != MPI_COMM_NULL)
	{
		MPI_Comm_free(&comm1);
		MPI_Group_free(&group1);
	}
	if(comm3 != MPI_COMM_NULL)
	{
		MPI_Comm_free(&comm3);
		MPI_Group_free(&group3);
		
	}
	
		
	if(comm2 != MPI_COMM_NULL)
	{
		MPI_Comm_free(&comm2);
		
	}
	
	
	MPI_Group_free(&group2);
	MPI_Group_free(&main_group);
	
	MPI_Finalize();
}



