/********************************************************************************	
*										*
*										*
*	Program  :: openmp_example18.c						*
*	Description :: Illustration of OpenMP 'teams' &'distribute' directives	*
*										*
*********************************************************************************/

#include<stdio.h>
#include<omp.h>

void main()
{
	int tid;
	/*
	 * 	NOTE:
	 *	i) teams directive must be contained within target construct and it is the first and only construct within target region.
	 *	ii)target construct must not contain any statements or directives outside teams directive.
	 */
	#pragma omp target
	{
		//*
		#pragma omp teams num_teams(2) thread_limit(3)
		{
			printf("Thread-%d of Team%d\n",omp_get_thread_num(),omp_get_team_num());
			
			/*
			 *	parallel, parallel sections, distribute, parallel loop, parallel loop simd 
			 *	are the only constructs closely nested within teams construct.
			 */
						
			printf("\n********* Team & Distribute ********\n\n");
			int i;
			#pragma omp distribute 
			for(i=0;i<3;i++)
			{
				printf("Team & Distribute: i =%d Thread-%d of Team-%d\n",i,omp_get_thread_num(),omp_get_team_num());				
			}

			// parallel sections can be but sections alone cannot be closely nested within teams directive
			#pragma omp parallel
			{
				#pragma omp sections
				{
					#pragma omp section
					{
						printf("Section-1 Thread-%d of Team%d\n",omp_get_thread_num(),omp_get_team_num());
					}
					#pragma omp section
					{
						printf("Section-2 Thread-%d of Team%d\n",omp_get_thread_num(),omp_get_team_num());
					}
				}
			}
		}//teams
	}//target
}
